#ifndef ENCODER_MOTOR_H
#define ENCODER_MOTOR_H

#include "Motor.hpp"
#include "SpeedEncoder.hpp"
#include "LerpSmoother.hpp"
#include "PIDController.hpp"
#include "SyncGroup.hpp"
#include <limits>

class EncoderMotor : public Motor
{
public:
    enum Mode
    {
        NoMode,
        SpeedMode,
        TurnMode
    };
    
    EncoderMotor(MotorData nData, EncoderData encData, PIDData speedPIDData, PIDData turnPIDData, SyncGroup *nSync = NULL);
    
    virtual void drive(float speed);
    virtual void rotate(float turn, float speedLimit = std::numeric_limits<float>::max());
    
    SpeedEncoder& getEncoder();
    float getSetSpeed() const;
    float getSetTurn() const;
    
//private:
    void tick();
    
    SpeedEncoder encoder;
    LerpSmoother speedSmoother;
    
    Mode mode;
    
    PIDController speedPID;
    float setSpeed;
    
    PIDController turnPID;
    float setTurn;
    float turnSpeedLimit;
    float errorTurn;
    
    SyncGroup *sync;
};

#endif // ENCODER_MOTOR_H